import 'reflect-metadata'
import express from "express"
import {Container} from "typedi"
import {UserService} from "../services/user-service"
import {body, check, validationResult,} from "express-validator"
import {hasValidationErrors, resultJson} from "../includes"
import {JWTUserPayload} from "../types"
import expressAsyncHandler from "express-async-handler"
import {UploadedFile} from "express-fileupload"

let userService = Container.get(UserService)

let userRouter = express.Router()
userRouter.post('/login',
    body('username').notEmpty().withMessage('请输入使用者名称'),
    body('password').notEmpty().withMessage('请输入密码'),
    expressAsyncHandler(async (req: express.Request, res: express.Response) => {
        if (hasValidationErrors(req, res)) return
        res.json(await userService.login(req.body.username, req.body.password))
    })
)
userRouter.get('/me', expressAsyncHandler(async (req: express.Request, res: express.Response) => {
    res.json(await userService.me(req.user as JWTUserPayload))
}))
userRouter.post('/uploadAvatar',
    expressAsyncHandler(async (req: express.Request, res: express.Response) => {
        if (!req.files?.image) {
            res.json(resultJson.error('请选择要上传的图片'))
            return
        }
        res.json(await userService.uploadAvatar(req.user as JWTUserPayload, req.files.image as UploadedFile))
    })
)
userRouter.post('/updateNickname',
    body('nickname').notEmpty().withMessage('请输入昵称').isLength({max: 31}).custom((input, meta) => {
        if (!/^[\u4e00-\u9fa5_a-zA-Z0-9]+$/.test(input)) {
            throw new Error('昵称只能包括中文，英文字母和数字及「_」')
        }
        return true
    }),
    expressAsyncHandler(async (req: express.Request, res: express.Response) => {
        if (hasValidationErrors(req, res)) return
        res.json(await userService.updateNickname(req.user as JWTUserPayload, req.body.nickname))
    })
)

export {userRouter}
